#!/bin/bash

# bwrep.sh: Generate short bandwidth usage report.
# This script will retrieve the bandwidth monitoring page from the ev1.net
# hosting service and reformat it for a simple text display to IRC.
# Script includes an optional daemon mode <-d> which will update this
# text file every hour.
# To customise the scripts output look at the generate_output function.
# Note that script requires at least GNU wget 1.9.1.  1.8.x will not work as
# it lacks the --post-data option required to submit the username and pass.

# Global variables and starting values.
username='RS-xxxx'
password='xxxx'
# This is the base directory for the script.
# Make sure you have write permissions for this directory.
bwrep_path="$HOME/eggdrop/bwrep"
# These are filenames in the base directory:
output_file="$bwrep_path/bwrep.txt"
html_file="$bwrep_path/bwrep.tmp.html"
pid_file="$bwrep_path/bwrep.pid"
log_file="$bwrep_path/bwrep.log" # set to /dev/null to disable logging.
# URL constants:
url='https://members.ev1.net/rsMembers/english'
# Initial values for program data variables.
total_up='0'
total_down='0'
total='0'
today='0'
quota='0'
quota_percent='0'
daily_avg='0'
last_updated='0'
daemon='0' # cmdline
ident='bwrep.sh (V0.5) Wed Jul 14 03:35:24 GMT 2004 jam/X'

function log {
# Simple logging function.
echo "$*" >> $log_file
return 0
}
# End function.
function parse_stats {
# This function will parse the html input
# Takes one argument-the input file.
local parse_line
if test ! -r "$1"
then
log 'Error reading html input.'
exit 1
fi
local parse_input="$1"
# Now we retrieve the actual data and store it in the correct variables.
# This data will need to be sanity checked before writing the file.
# Note that the -A option to grep below says to return X many lines after the
# match is found.  I did this because the lines we want to grab don't have much
# unique data to pick out with grep.
parse_line=`grep -A 4 '(Down from Server)' "$parse_input" |tail -n1`
total_up=`echo "$parse_line" |cut -d '>' -f3 |cut -d '<' -f1`
reset_fields
parse_line=`grep -A 5 '(Down from Server)' "$parse_input" |tail -n1`
total_down=`echo "$parse_line" |cut -d '>' -f3 |cut -d '<' -f1`
reset_fields
parse_line=`grep 'Total (In + Out) Bandwidth:' "$parse_input"`
total=`echo $parse_line |cut -d '>' -f3 |cut -d '<' -f1`
reset_fields
parse_line=`grep -A 3 'Daily Average for<br>Current Month' "$parse_input" \
|tail -n1`
today=`echo "$parse_line" |cut -d '>' -f3 |cut -d '<' -f1`
reset_fields
parse_line=`grep -A 4 'Daily Average for<br>Current Month' "$parse_input" \
|tail -n1`
daily_avg=`echo "$parse_line" |cut -d '>' -f3 |cut -d '<' -f1`
reset_fields
parse_line=`grep 'Your Monthly Bandwidth Quota is:' "$parse_input"`
quota=`echo $parse_line |cut -d '>' -f2 |cut -d '<' -f1`
reset_fields
parse_line=`grep 'You have used' "$parse_input"`
quota_percent=`echo $parse_line |cut -d '>' -f2 |cut -d '<' -f1`
reset_fields
parse_line=`grep 'Last Updated: ' "$parse_input"`
last_updated=`echo $parse_line |cut -d '>' -f2 |cut -d '<' -f1`
reset_fields
log 'Parsed html file.'
return 0
} 
# End function.
function reset_fields {
# Reset the fields of the parser
# This helps with error checking if called correctly.
# helper function for parse_stats
parse_line=''
return 0
}
# End function.
function check_stats {
# This function sanity checks the data retrieved by parse_stats
# This function should always be called before writing out the final report.
if [ "$total" = '' -o "$total" = '0' ]
then
log 'Error retrieving total.'
return 1
fi
if [ "$total_up" = '' -o "$total_up" = '0' ]
then
log 'Error retrieving total uploads.'
return 1
fi
if [ "$total_down" = '' -o "$total_down" = '0' ]
then
log 'Error retrieving total downloads.'
return 1
fi
if [ "$today" = '' -o "$today" = '0' ]
then
log 'Error retrieving bandwidth usage for today.'
return 1
fi
if [ "$quota_percent" = '' -o "$quota_percent" = '0' ]
then
log 'Error retrieving used quota percentage.'
return 1
fi
if [ "$quota" = '' -o "$quota" = '0' ]
then
log 'Error retrieving  quota.'
return 1
fi
if [ "$daily_avg" = '' -o "$daily_avg" = '0' ]
then
log 'Error retrieving daily average.'
return 1
fi
if [ "$last_updated" = '' -o "$last_updated" = '0' ]
 then
log 'Error retrieving last update time.'
return 1
fi
return 0
}
# End function.
function get_stats_data {
# This should retrieve the stats html from the web hosting company
rm -f "$html_file"
if ! touch "$html_file"
then
log 'Error writing to temp html file.'
exit 1
fi
if ! test -w "$html_file"
then
log 'Error writing to temp html file.'
return 1
fi
if wget-1.9.1 -q -O $html_file \
--post-data "Rack.Bill.Cust_LoginID=${username}&Rack.Bill.Cust_LoginPass=${password}&Submit=Submit" \
--save-headers $url/index.asp $url/bandwidth.asp
then
return 0
else
log 'wget returns error.'
return 1
fi
return 1
}
# End function
function generate_output {
# Generates the final report.
# this kills the old report so the data had better be sanity checked.
rm -f $output_file
if ! touch "$output_file"
then
 log 'Error writing to output file.'
return 1
fi
if test ! -w "$output_file"
then
log 'Error writing to output file.'
return 1
fi
echo -e "00[14lna00]14 - Usage reports indicate that 04${quota_percent} 14of the \
00${quota} 14quota has been used, \
total used 00${total}14, 00${total_up} 14OUT14, 00${total_down} 14IN14, \
00${today} 14today. The daily average is 00${daily_avg}14. \
Stats last updated: 00${last_updated}" > $output_file
 log 'Report generation complete.'
return 0
}
# End function.
function parse_cmdline {
# Parse command line options-only a few right now.
while [ "$1" != '' ]
do
if [ "$1" = '--help' -o "$1" = '-h' ]
then
usage_info
exit 0
elif [ "$1" = '--version' ]
then
 echo "$ident"
exit 0
elif [ "$1" = '-d' ]
then
daemon='1'
shift
else
echo "Invalid command line option: $1"
exit 1
fi
 done
return 0
} 
# End function.
function usage_info {
local prgname=`basename $0`
echo "$ident"
echo
echo "Usage: $prgname [-d]"
echo '-d: Run as a daemon and update every hour.'
exit 0
}
# End function.
function run_daemon {
# Runs this script as a daemon until sent a kill signal.
sleep 1
log 'Running as a daemon'
while test ! -e $bwrep_path/bwrep.exit
do
$0
log 'sleeping for 3600 seconds.'
sleep 3600
done
exit 0
}
## End function.

# Execution starts here
parse_cmdline $*
if [ "$daemon" = '1' ]
then
if [ "$log_file" != '/dev/null' ]
then
rm -f $log_file
fi
# first if we have a pid file try to kill off that old process.
if test -r "$pid_file"
then
old_pid=`cat "$pid_file"`
log "Trying to kill old process: $old_pid"
kill $old_pid
fi
rm -f $pid_file
run_daemon >& /dev/null &
echo "$!" > $pid_file
exit 0
fi
log "Program started `date`"
if ! get_stats_data
then
exit 1
fi
parse_stats "$html_file"
if check_stats
then
if generate_output
then
log 'success'
exit 0
else
log 'Error generating output.'
exit 1
 fi
else
log 'Error parsing html file.'
exit 1
fi
